/*****************************  Bipede   ******************************
**								     **
** File           : $HOME/BIP/utils_vxworks/Socket/comBip.c	     **
** Author         : RPG, Soraya Arias and Sbastien Jard	     **
** Version        : 1.1 					     **
** Creation       : 1 Juillet 2002				     **
**								     **
******************************* Description ***************************
**								     **
**  Gestion par string de la communication pour le calcul de 	     **
**  la loi de commande du robot BIP			     	     **
**								     **
****************************** Modifications **************************
**								     **
** PBPBPB : affectation de la structure combip desc !!		     **
**								     **
***********************************************************************
** (c) Copyright 2000, INRIA, all rights reserved		     **
**********************************************************************/
#ifndef __combip_c
#define __combip_c

#include <stdio.h>
#include <string.h>
#include "comBip.h"

#define PORT 5001
#define MAX_SIZE_MESSAGE 1024
#define SIZE_STR 1024

//dimension du vecteur des contacts actifs
#define DIM_C 24 

#define Q_VECTTYPE 0
#define QDOT_VECTTYPE 1
#define LAMBDA_VECTTYPE 2
#define COM_VECTTYPE 3

#define COMBIPSTATUS_NEW 0
#define COMBIPSTATUS_OLD 1

/****************************/
/* Combip Structure created */
/****************************/
static COMBIP_DESC bipVect;

/******************/
/* Socket created */
/******************/
static SOCK_SERVER s;
SOCK_SERVER *combipSocket() 
{
  return &s;
}

/*****************************/
/* Create a combip structure */
/*****************************/
int combipStructCreate(COMBIP_DESC *combipStruct, int dim)
{
 combipStruct->dimrow = dim;
 combipStruct->lambda = (double *)malloc(DIM_C*sizeof(double));
 combipStruct->q = (double *)malloc(dim*sizeof(double));
 combipStruct->qdot = (double *)malloc(dim*sizeof(double));
 combipStruct->commande = (double *)malloc(dim*sizeof(double));
 combipStruct->status = COMBIPSTATUS_OLD;

  return OK;
}

/************************************************************/ 
/* Return a Commande vector pointer from a combip structure */
/************************************************************/ 
void *combipGetComPointer(COMBIP_DESC *combipStruc)
{
  if (combipStruc == NULL) {
    fprintf(stderr, "combipGetPointer***: arg combipStruc NULL !\n");
    return(NULL);
  }
  return(combipStruc->commande);
}

/*****************************************************/
/* Return a q vector pointer from a combip structure */
/*****************************************************/
void *combipGetqPointer(COMBIP_DESC *combipStruc)
{
  if (combipStruc == NULL) {
    fprintf(stderr, "combipGetPointer***: arg combipStruc NULL !\n");
    return(NULL);
  }
  return(combipStruc->q);
}

/*************************************************************/
/* Convert a vector from the combip structure into a string  */
/*************************************************************/ 
int convertCombipVect2String(double *combipVect,char *vect2Str,int dim)
{
  int i;
  char *pvect = vect2Str;

  for(i = 0; i < dim; i++) {
    sprintf(pvect,"%lf:", combipVect[i]); 
    pvect = vect2Str + strlen(vect2Str);
  }
  
  return OK;
}

/*********************************************************************/ 
/* Convert a string received through the socket into a combip vector */
/*********************************************************************/ 
int convertString2CombipVect(double *combipVect,char *str2Vect,int dim)
{
  int i;
  char *pstr = (char *)malloc((sizeof(char))*strlen(str2Vect)+1);

  for(i = 0; i < dim ; i++) {
    if (i == 0) {
      pstr = strtok(str2Vect,":");
    }
    else {
      pstr = strtok(NULL,":");
    }

    sscanf(pstr,"%lf", &(combipVect[i])); 
  }
  return OK;
}

/***********************************************************/
/* Send combip vector as message string through the socket */
/***********************************************************/
int combipSockSendVect(COMBIP_DESC *bipVect, int vectType)
{
  char vect2Str[SIZE_STR];

  if (bipVect != NULL) {
    switch (vectType) {
    case Q_VECTTYPE: {
      convertCombipVect2String(bipVect->q, vect2Str, bipVect->dimrow);
      break;
    }
    case QDOT_VECTTYPE: {
      convertCombipVect2String(bipVect->qdot,vect2Str,bipVect->dimrow);
      break;
    }
    case LAMBDA_VECTTYPE: {
      convertCombipVect2String(bipVect->lambda, vect2Str, DIM_C);
      break;
    }
    case COM_VECTTYPE: {
      convertCombipVect2String(bipVect->commande, vect2Str, bipVect->dimrow);
      break;
    }
    default: {
      printf("Vector Type not supported \n");
      return ERROR;
    }
  }

    return(sockSendString(combipSocket(), vect2Str)) ;   
  }
  else {
    printf("combipSockSend combip structure error ! \n");
    return ERROR;
  }
  return OK;
}

/**************************************************************/
/* Receive combip vector as message string through the socket */
/**************************************************************/
int combipSockRecVect(COMBIP_DESC *bipVect, int vectType)
{
  char str2Vect[SIZE_STR];

    if (sockRec(combipSocket(), str2Vect, MAX_SIZE_MESSAGE) == ERROR)
    {
      return ERROR;
    }

  if (bipVect != NULL) {
    switch (vectType) {
    case Q_VECTTYPE: 
      convertString2CombipVect(bipVect->q, str2Vect, bipVect->dimrow);
      break;
    case QDOT_VECTTYPE: 
      convertString2CombipVect(bipVect->qdot,str2Vect,bipVect->dimrow);
      break;
    case LAMBDA_VECTTYPE:
      convertString2CombipVect(bipVect->lambda, str2Vect, DIM_C);
      break;
    case COM_VECTTYPE:
      convertString2CombipVect(bipVect->commande, str2Vect, bipVect->dimrow);
      break;
    default:
      printf("Vector Type not supported \n");
      return ERROR;
    }   
  }
  else {
    printf("combipSockRec combip structure error ! \n");
    return ERROR;
  }
  return OK;
}

/********************************/
/* Send q Vector through socket */
/********************************/
int combipSockSendq(COMBIP_DESC *bipVect) 
{ 
  if(combipSockSendVect(bipVect, Q_VECTTYPE) == ERROR) {
    printf("combipSockSendq -> Error sending q vector through socket, error = %d, OK= %d \n", ERROR, OK);
    return ERROR;
  }

  return OK;
}

/***********************************/
/* Send qdot Vector through socket */
/***********************************/
int combipSockSendqdot(COMBIP_DESC *bipVect) 
{
  if(combipSockSendVect(bipVect, QDOT_VECTTYPE) == ERROR) {
    printf("combipSockSendqdot -> Error sending qdot through socket , error = %d, OK= %d \n", ERROR, OK);
    return ERROR;
  }

  return OK;
}

/*************************************/
/* Send Lambda Vector through socket */
/*************************************/
int combipSockSendLambda(COMBIP_DESC *bipVect) 
{
  if(combipSockSendVect(bipVect, LAMBDA_VECTTYPE) == ERROR) {
    printf("combipSockSendLambda -> Error sending Lambda vector through socket \n");
    return ERROR;
  }

  return OK;
}

/***************************************/
/* Send commande Vector through socket */
/***************************************/
int combipSockSendCom(COMBIP_DESC *bipVect) 
{
  if(combipSockSendVect(bipVect, COM_VECTTYPE) == ERROR) {
    printf("combipSockSendCom -> Error sending Commande vector through socket \n");
    return ERROR;
  }

  return OK;
}

/***********************************/
/* Receive q Vector through socket */
/***********************************/
int combipSockRecq(COMBIP_DESC *bipVect)
{
  if(combipSockRecVect(bipVect, Q_VECTTYPE) == ERROR) {
    printf("combipSockSendq -> Error receiving q vector through socket \n");
    return ERROR;
  }

  return OK;
}
/**************************************/
/* Receive qdot Vector through socket */
/**************************************/
int combipSockRecqdot(COMBIP_DESC *bipVect)
{
  if(combipSockRecVect(bipVect, QDOT_VECTTYPE) == ERROR) {
    printf("combipSockSendqdot -> Error receiving qdot vector through socket \n");
    return ERROR;
  }

  return OK;
}

/****************************************/
/* Receive Lambda Vector through socket */
/****************************************/
int combipSockRecLambda(COMBIP_DESC *bipVect)
{
  if(combipSockRecVect(bipVect, LAMBDA_VECTTYPE) == ERROR) {
    printf("combipSockRecLambda -> Error receiving lambda vector through socket \n");
    return ERROR;
  }

  /* Change combip structure status */
  bipVect->status = COMBIPSTATUS_NEW;

  return OK;
}

/******************************************/
/* Receive Commande Vector through socket */
/******************************************/
int combipSockRecCom(COMBIP_DESC *bipVect)
{
  if(combipSockRecVect(bipVect, COM_VECTTYPE) == ERROR) {
    printf("combipSockRecCom -> Error receiving Commande vector through socket \n");
    return ERROR;
  }

  /* Change combip structure status */
  bipVect->status = COMBIPSTATUS_NEW;

  return OK;
}

/*************************/
/* Ouvre 1 socket client */
/*************************/
int combipOpenClient(char *target)
{
 int tempo=0;
 char message[MAX_SIZE_MESSAGE];

 while (sockClientOpen(&s, target, PORT) == ERROR) {
    sleep(2); /* time for vxworks to run the sockServer */
    tempo++;
    /* Temporisation */
    if (tempo > 10) {
      fprintf(stderr,"sock: Give up...\n");
       return(ERROR);
    } else {
      fprintf(stderr,"sock: Retrying...\n");
    }
  }
#ifdef DEBUG
  printf("connection ok\n");
#endif

  if (sockIsAlive(&s) == ERROR) {
    printf("problem socket...closing\n");
    sockClose(&s);
    return ERROR;
  } else {
    sockRec(&s, message, MAX_SIZE_MESSAGE);
    printf("message received <%s>\n", message);
  }
  return OK;
}

/*****************/
/* socket closed */
/*****************/
int combipClose()
{
  sockClose(&s);
  return OK;
}

#endif
