#define _POSIX_C_SOURCE 199506

#include <pthread.h>
#include <limits.h>
#include <stdio.h>
#include <unistd.h>

#include "comBip.h"

#define NBLOOP 1000
 
typedef void (*FUNCPTR)(void *);
//special type for cast in thr_create()
typedef void * (*SOSO)  (void *); 

void modele_gravite(double G[21],double q[21]);
void jacobien_contact(double q[21], double J[21][24]);
void inertie(double q[21], double M[21][21]);
void contact(double q[21], double C[1][24]);
void sortie(double q[21], double T[1][21]);
void jacobien_sortie(double q[21], double J[21][21]);
void hessiens_sortie(double q[21], double qdot[21], double H[1][21]);

COMBIP_DESC bipVect;
 
int orcSpawn(pthread_t *tid, FUNCPTR funcptr, void * arg)
{
  int status;
  pthread_attr_t attributes;
 
  // Allocation and initialization of the attribute structure
  pthread_attr_init(&attributes);
  
  //create pthread 
  status =  pthread_create(tid, &attributes, (SOSO)funcptr, arg);
  printf("pthread_create error = %d \n", status);
 
  return status;
}

/*****************************************/
/* Server Program wait for vectors from  */
/* the client and comput the control law */
/*****************************************/
int combipServ()
{
  int k,i,l,j;
  double q[21], qdot[21], J_C[21][24], M[21][21], 
  	 		  H_SH[21], M_PD[21], C_LAMBDA[21], 
			  T[1][21], J_S[21][21], H[1][21],
			  s_hs[21], s_T[21];
			  
  /* Create the Bip Struct -> vector dimension = 21 */
  combipStructCreate(&bipVect,21);

  /* Server opens a socket */
  if (combipOpen() != 0) {
    printf("Server can not open socket \n");
    return ;
  }

  /* compute with q, qdot and lambda in input and Commande in output */
  for ( k = 0 ; k < NBLOOP ; k++) 
  {
    printf("receive q, qdot and Lambda vector...!\n");
    combipSockRecq(&bipVect);
    combipSockRecqdot(&bipVect);
    combipSockRecLambda(&bipVect);

    /* Computation */
    	printf("compute...!\n");
    
    /* somme sortie(q) - hessiens(q) */
  	sortie(bipVect.q, T);
	hessiens_sortie(bipVect.q, bipVect.qdot, H);
	for(l=0;l<bipVect.dimrow;l++) s_hs[l] = T[1][l] - H[1][l] ;
	
    /* multiplication H(q)*[sortie(q) - hessiens(q)] */
	jacobien_sortie(bipVect.q, J_S);
	for(l=0;l<21;l++)H_SH[l]=0;
	for(i=0;i<bipVect.dimrow;i++)
  	{
  		for(j=0;j<bipVect.dimrow;j++)
		{
			H_SH[i] = H_SH[i] +J_S[i][j]*s_hs[j];
		}
  	}
    /* multiplication M(q)*H(q)*[sortie(q) - hessiens(q)] */
  	for(l=0;l<21;l++)M_PD[l]=0;
	inertie(bipVect.q, M);
  	for(i=0;i<bipVect.dimrow;i++)
  	{
  		for(j=0;j<bipVect.dimrow;j++)
		{
			M_PD[i] = M_PD[i] +M[i][j]*H_SH[j];
		}
  	}
	
    
    /* multiplication C(q)*lambda */
    for(l=0;l<21;l++)C_LAMBDA[l]=0;
    jacobien_contact(bipVect.q, J_C);
    for(i=0;i<bipVect.dimrow;i++)
    {
        for(j=0;j<24;j++)
        {
                C_LAMBDA[i] = C_LAMBDA[i] +J_C[i][j]*bipVect.lambda[j];
        }
    }
    
    /* M(q)*H(q)*[sortie(q) - hessiens(q)] + C(q)*lambda */
    for(l=0;l<bipVect.dimrow;l++)
    {
    	s_T[l] = M_PD[l] + C_LAMBDA[l];
	bipVect.commande[l] = s_T[l];
    }
	
    /* Send Message including result G to Client */
    printf("send Commande vector...!\n");
    combipSockSendCom(&bipVect);
  }

  printf("closing socket\n");
  combipClose();
  return(0); 
}


int main(int argc, char **argv)
{
  int n;
  pthread_t ServManageId;

  if (orcSpawn(&ServManageId,(FUNCPTR) combipServ, NULL) == ERROR)
  {
    printf("ERROR:Can't create thread from the server management \n");
    return(1);
  }
  
  printf("orcSpawn called, entering pthread_join \n");

  /* Suspend the main thread on the terminaison of the ServManageId thread */
  if (n = pthread_join(ServManageId, NULL)) {
    fprintf(stderr, "pthread_join: %s\n", strerror(n));
    exit(1);
  }
  
  printf("Main combipservtest end. \n");

  return(0);
}
