#include <stdio.h>
#include <math.h>
#include <taskLib.h>

#include "hardBip.h"
#include "drvBip.h"

/**************************************************************/
/* nombre d'essai pour le calcul de frottement sec, la valeur */
/* max sur les moteurs est donc NB_ESSAI * INCR_CONSIGNE      */
/**************************************************************/
#define NB_ESSAI 80 /*20*/

/* temps d'attente au demarrage apres consigne moteur en tick */
#define T_D_MOTEUR 30

/* en tick, temps d'attente a l'arret apres consigne moteur */
#define T_F_MOTEUR 20

/* nombre de pas codeurs pour valider que le moteur a bouge */
#define DELTA_CODEUR 1.5

/* increment sur la consigne de commande */
#define INCR_CONSIGNE 0.03

/* variable gerant l'initialisation */
static int bipTestInitFlag = ERROR;

#define BIP_DOF 15

int bipTestInit()
{
  if (bipTestInitFlag == ERROR)
    {  
      bipHardInit();
      bipTestInitFlag = OK;
    }
  return(OK);
}
static int bipTestFrottement(int sens,int index)
{
  int c, cc;
  float consigne;
  int fin_test = 0;
  double current[BIP_DOF];
  double joint_init[BIP_DOF];
  double joint[BIP_DOF];
  
  if ((sens != 1) && (sens != -1))
    {
      printf("Pb Argument\n");
      return(-1);
    }

  /* gestion de l'init */
  if (bipTestInitFlag == ERROR)
    {  
      bipHardInit();
      bipTestInitFlag = OK;
    }

  /* init */
  printf("init variables\n");
  for(c=0;c<BIP_DOF;c++)
  	{
	current[c]=0.0;
	}
  bipPutCurrent(current);
  bipGetEncoders(joint_init);
  
  
  printf("Moteur No %d\n", index);
  fin_test = 0;
  /* increment de sortie: jusqu'a ce que ca bouge */
  for (c=1;(c<=NB_ESSAI) && (fin_test==0);c++)
	{
	  current[index]=INCR_CONSIGNE*sens*c;
	  printf("consigne=%.2f\n", current[index]);
	  bipPutCurrent(current);
	  
	  /* Demarage du moteur */
	  taskDelay(T_D_MOTEUR);
	  
	  /* arret moteur, les donnees sont lues */
	  current[index]=0.0;
	  bipPutCurrent(current);
	  
	  /* prise en compte de l'inertie moteur pour l'arret */
	  taskDelay(T_F_MOTEUR);
	  
	  /* les donnees sont lues */
	  bipGetEncoders(joint);
	  printf("codeur %d : %f\n", index, joint[index] - joint_init[index]);
	  
	  if (fabs(joint[index] - joint_init[index]) > DELTA_CODEUR)
		{ 
		  fin_test = 1;
		  consigne = INCR_CONSIGNE*sens*c;
		}
          for(cc=0;cc<BIP_DOF;cc++)
        	{
                  joint_init[cc] = joint[cc];
                }
	}
    
  printf("Moteur No %d : fs= %.2f\n",index, consigne);
    
  return(0);
}


int bipTestFrotPos(int index)
{
  return(bipTestFrottement(1,index));
}


int bipTestFrotNeg(int index)
{
  return(bipTestFrottement(-1,index));
}
